"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getType = exports.createParam = exports.createRequestResponse = exports.createQueryParams = exports.createPath = exports.createCall = exports.createApi = void 0;
const utils_1 = require("./utils");
const serializefunc_1 = require("./serializefunc");
const createApi = (models, bundle) => {
    return `/* tslint:disable */
/* eslint-disable */
    
import * as models from './models';
import * as runtime from './runtime';
import {${serializefunc_1.THROW_NULLABLE_ERROR_METHOD_NAME}} from './models'; 
    
${bundle.calls.map(exports.createRequestResponse).join('\n')}
     
export class ${(0, utils_1.pascalCase)(bundle.name)}API extends runtime.BaseAPI {
    ${bundle.calls.map(call => (0, exports.createCall)(models, call)).join('\n    ')}
}
`;
};
exports.createApi = createApi;
const createCall = (models, call) => {
    return `
    async ${call.nickname}Raw(request: ${(0, utils_1.pascalCase)(call.nickname)}Request${call.body ? ', body: ' + (0, exports.getType)(call.body) : ''}): Promise<runtime.ApiResponse<${(0, utils_1.pascalCase)(call.nickname)}Response>> {
        const queryParameters: runtime.HTTPQuery = {};
        ${(0, exports.createQueryParams)(call)}
        const headerParameters: runtime.HTTPHeaders = {};
        headerParameters['Content-Type'] = 'application/json';

        const response = await this.request({
            path: ${(0, exports.createPath)(call)},
            method: '${call.method}',
            headers: headerParameters,
            query: queryParameters,
            ${call.body ? ('body: ' + (0, utils_1.property)(models, call.body, true, 'body', 'models.') + ',') : ''}
        });

        return new runtime.${call.returnType ? 'JSON' : 'Void'}ApiResponse${call.returnType ? '<any>' : ''}(response${call.returnType ? ', (value) => ' + (0, utils_1.property)(models, call.returnType, false, 'value', 'models.') : ''});
    }
    async ${call.nickname}(request: ${(0, utils_1.pascalCase)(call.nickname)}Request${call.body ? ', body: ' + (0, exports.getType)(call.body) : ''}): Promise<${(0, utils_1.pascalCase)(call.nickname)}Response> {
        const response = await this.${call.nickname}Raw(request${call.body ? ', body' : ''});
        return await response.value();
    }
    `;
};
exports.createCall = createCall;
const createPath = (call) => {
    if (!call.params.some(param => param.in === 'path')) {
        return `'${call.path}'`;
    }
    return `'${call.path}'.` + call.params.filter(param => param.in === 'path').map((param) => `replace("{${param.name}}", encodeURIComponent(request['${param.interfaceName}']))`).join('.');
};
exports.createPath = createPath;
const createQueryParams = (call) => {
    const requiredStr = (param) => (param.required ? '' : `if (request.${param.interfaceName} !== undefined)`);
    return call.params.filter(param => param.in === 'query').map(param => requiredStr(param) + `queryParameters['${param.name}'] = request['${param.interfaceName}'];`)
        .join('\n        ');
};
exports.createQueryParams = createQueryParams;
const createRequestResponse = (call) => {
    return `
export interface ${(0, utils_1.pascalCase)(call.nickname)}Request {
    ${call.params.map(exports.createParam).join('\n    ')}
}

export type ${(0, utils_1.pascalCase)(call.nickname)}Response = ${(0, exports.getType)(call.returnType)};
`;
};
exports.createRequestResponse = createRequestResponse;
const createParam = (params) => `${params.interfaceName}${params.required ? ':' : '?:'} ${(0, exports.getType)(params.type)};`;
exports.createParam = createParam;
const getType = (s) => {
    if (!s) {
        return 'void';
    }
    return utils_1.primitives.some((o) => s.startsWith(o)) ? s : 'models.' + s;
};
exports.getType = getType;
//# sourceMappingURL=templateApi.js.map