"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnionBaseSuffix = exports.resolveReference = exports.pascalCase = exports.refToName = exports.primitives = exports.property = void 0;
const tslib_1 = require("tslib");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const path_1 = tslib_1.__importDefault(require("path"));
const serializefunc_1 = require("./serializefunc");
const property = (models, s, serialize, name, prefix = '') => {
    var _a, _b;
    const type = typeof s === 'string' ? s : (_b = (_a = s.getTypeNode()) === null || _a === void 0 ? void 0 : _a.getText()) !== null && _b !== void 0 ? _b : 'unknown';
    const isRequired = typeof s !== 'string' && !s.hasQuestionToken();
    if (type === null || type === void 0 ? void 0 : type.endsWith('[]')) {
        return `${name}?.map((inner: any) => ${singleProperty(models, type === null || type === void 0 ? void 0 : type.slice(0, -2), serialize, 'inner', prefix, isRequired)})`;
    }
    return `${singleProperty(models, type, serialize, name, prefix, isRequired)}`;
};
exports.property = property;
const defaultValueForType = (name, type) => {
    if (type === 'string') {
        return '""';
    }
    if (type === 'number') {
        return '0';
    }
    return undefined;
};
const singleProperty = (models, type, serialize, name, prefix, isRequired) => {
    if (type === 'Date') {
        return serialize ? `${name}?.toISOString()` : `${name} ? new Date(${name}) : ${name}`;
    }
    if (exports.primitives.includes(type)) {
        return isRequired && !serialize ? `${serializefunc_1.REQUIRE_NOT_NULLABLE_METHOD_NAME}(${name}, ${defaultValueForType(name, type)})` : name;
    }
    if (type.endsWith("'") && type.startsWith("'")) {
        return type;
    }
    if (type.endsWith(exports.UnionBaseSuffix) && models.getTypeAlias(type.slice(0, -(exports.UnionBaseSuffix.length)))) {
        type = type.slice(0, -(exports.UnionBaseSuffix.length));
    }
    const callToNextLevelDeSerializationAsString = prefix + type + (serialize ? 'Serialize' : 'Deserialize') + `(${name} as any)`;
    const handleUndefinedAsString = !isRequired || serialize ? name : serializefunc_1.THROW_NULLABLE_ERROR_METHOD_NAME + '()';
    return `${name} !== undefined && ${name} !== null ? ${callToNextLevelDeSerializationAsString} : ${handleUndefinedAsString}`;
};
exports.primitives = ['string', 'boolean', 'number', 'void', 'undefined', 'unknown', 'never', 'any', 'Date'];
const refToName = ($ref) => {
    const typeName = lodash_1.default.last($ref.split("/"));
    if (typeName === undefined) {
        throw new Error('illegal schema ' + $ref + " ref has no name");
    }
    return typeName.replace('.yml', '');
};
exports.refToName = refToName;
const pascalCase = (value) => {
    const snake = lodash_1.default.camelCase(value);
    return snake[0].toUpperCase() + snake.slice(1);
};
exports.pascalCase = pascalCase;
const resolveReference = (ctx, $ref, cb) => {
    if ($ref.startsWith("#/")) {
        return cb({
            newCtx: Object.assign(Object.assign({}, ctx), { refPath: ctx.refPath }),
            name: (0, exports.refToName)($ref),
            resolved: ctx.resolver.get(ctx.currFile + $ref)
        });
    }
    const newPath = $ref.split('#')[0].split('/').slice(0, -1).join('/');
    const file = path_1.default.join(...[...ctx.refPath, $ref]);
    return cb({ newCtx: Object.assign(Object.assign({}, ctx), { refPath: ctx.refPath.concat(newPath), currFile: file.split('#')[0] }), name: (0, exports.refToName)($ref), resolved: ctx.resolver.get(file) });
};
exports.resolveReference = resolveReference;
exports.UnionBaseSuffix = 'Base';
//# sourceMappingURL=utils.js.map